import logging.handlers
import pika
import os
from pika.adapters.blocking_connection import BlockingConnection, BlockingChannel
import json


def get_logging_level():
    loglevel = int(os.environ.get('loglevel', 0))
    level = logging.INFO
    if loglevel == 0:
        level = logging.DEBUG
    elif loglevel == 1:
        level = logging.INFO
    elif loglevel == 2:
        level = logging.WARNING
    elif loglevel == 3:
        level = logging.ERROR
    elif loglevel == 4:
        level = logging.CRITICAL
    else:
        level = logging.INFO
    return level


class RabbitMQHandler(logging.Handler):

    def __init__(self, level=None):

        if not level:
            level = get_logging_level()

        super().__init__(level)
        fmt = ('%(asctime)s$$%(levelname)s$$%(name)s$$%(threadName)s$$'
               '%(module)s.%(funcName)s:%(lineno)4d$$%(message)s')
        fmt = logging.Formatter(fmt, None, "%")
        self.setFormatter(fmt)

        self.rabbithost = os.environ.get('rabbithost', 'localhost')
        self.rabbitport = int(os.environ.get('rabbitport', 5672))
        self.connection: pika.BlockingConnection = None
        self.channel: BlockingChannel = None
        self.exchange = "logs"
        self.open_connection()
        self.createLock()

    def open_connection(self):
        try:
            if not self.connection or self.connection.is_closed:
                self.connection = pika.BlockingConnection(
                    pika.ConnectionParameters(host=self.rabbithost, port=self.rabbitport))
            self.channel = self.connection.channel()
            self.channel.exchange_declare(
                exchange=self.exchange, exchange_type="topic", durable=True, auto_delete=False)
            return True
        except:
            self.connection = None
            self.channel = None
            return False

    def close_connection(self):
        if self.channel and not self.channel.is_closed:
            self.channel.close()

        if self.connection and not self.connection.is_closed:
            self.connection.close()

        self.connection, self.channel = None, None

    def emit(self, record):
        try:
            openflg = True
            self.acquire()
            if not self.connection or self.connection.is_closed or not self.channel or self.channel.is_closed:
                openflg = self.open_connection()
            if openflg:
                routing_key = record.levelname
                formatted = self.format(record)
                formatted = formatted.split('$$')
                json_data = {
                    'time': formatted[0],
                    'level': formatted[1],
                    'logname': formatted[2],
                    'threadname': formatted[3],
                    'modulename': formatted[4],
                    'message': formatted[5]
                }

                self.channel.basic_publish(
                    exchange=self.exchange, routing_key=routing_key, body=json.dumps(json_data))
        except Exception:
            self.connection, self.channel = None, None
            # self.handleError(record)
        finally:
            self.release()

    def close(self):
        self.acquire()

        try:
            self.close_connection()
        finally:
            self.release()

    def __del__(self):
        self.close()



from setuptools import setup
import pathlib

_ROOT = pathlib.Path(__file__).parent

with open(str(_ROOT / 'bdlogging' / '__init__.py')) as f:
    for line in f:
        if line.startswith('__version__ ='):
            _, _, version = line.partition('=')
            VERSION = version.strip(" \n'\"")
            break
        else:
            raise RuntimeError(
                'unable to read the version from bdlogging/__init__.py')


requires = ["pika"]
setup(
    name='bdlogging',
    version=VERSION,
    description='RabbitMQログ出力モジュール',
    url='https://gitlab.com/belldata/logging.git',
    author='nozomi.nishinohara',
    author_email='nozomi.nishinohara@belldata.co.jp',
    # license='Apache License, Version 2.0',
    keywords='',
    packages=[
        "bdlogging"
    ],
    install_requires=requires,
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
)



__version__ = '0.0.8'


from .handlers import RabbitMQHandler, get_logging_level
from logging import Logger, root
import logging
import time


def getLogger_add_rabbithandler(name=None, level=None):
    """
    Return a logger with the specified name, creating it if necessary.

    If no name is specified, return the root logger.
    """
    # cnt = 0
    # if not level:
    #     level = get_logging_level()
    # rabbit: RabbitMQHandler = None
    # while True:
    #     try:
    #         rabbit = RabbitMQHandler(level)
    #         break
    #     except Exception as ex:
    #         if cnt > 10:
    #             raise ex
    #         time.sleep(5)
    #         cnt += 1

    logger = None
    if name:
        logger = Logger.manager.getLogger(name)
    else:
        logger = root
    logger.setLevel(level)
    # logger.addHandler(rabbit)
    return logger


__all__ = ["__version__", "RabbitMQHandler", "getLogger_add_rabbithandler"]
